import 'package:nexstate/nexstate.dart';
import 'package:test/test.dart';

void main() {
  test('Manages State Correctly', () async {
    final userStore = UserStore();

    const user = User(
      firstName: 'John',
      lastName: 'Doe',
      userName: 'john_doe',
    );

    await userStore.dispatch(const LogIn(user));
    expect(userStore.state, user);

    await userStore.dispatch(const LogOut());
    expect(userStore.state, null);
  });
}

class User {
  final String firstName;
  final String lastName;
  final String userName;

  const User({
    required this.firstName,
    required this.lastName,
    required this.userName,
  });

  @override
  String toString() {
    return {
      'firstName': firstName,
      'lastName': lastName,
      'userName': userName,
    }.toString();
  }
}

class UserStore extends Store<User?> {
  UserStore() : super(null);
}

class LogIn extends Action<User?> {
  final User user;

  const LogIn(this.user) : super();

  @override
  Future<User> activate(User? state) async => user;
}

class LogOut extends Action<User?> {
  const LogOut() : super();

  @override
  // ignore: prefer_void_to_null
  Future<Null> activate(User? state) async => null;
}
